#!/usr/local/bin/perl

#
# datetime.pl
#
# CurrentShortDate()
# CurrentTime()
# FormatDaemonDateStr()
# FormatDateStr()
# FormatDateTimeStr()
# FormatTimeStr()
#

###############################################################################
# Procedure:   CurrentShortDate
#
# Description: Returns the current date, formatted according to the configuration
#              file.
#
# Parameters:  (n/a)
#
# Return vals: the current date
###############################################################################
sub CurrentShortDate {
	$mday = sprintf( "%02d", (localtime)[3]);
	$mon = sprintf( "%02d", (localtime)[4]+1);
	$year = (localtime)[5] + 1900;

    return FormatDateStr("$year$mon$mday");
}

###############################################################################
# Procedure:   CurrentTime
#
# Description: Returns the current time, formatted according to the configuration
#              file.
#
# Parameters:  (n/a)
#
# Return vals: the current time
###############################################################################
sub CurrentTime {
    $sec  = sprintf( "%02d", (localtime)[0]);
    $min  = sprintf( "%02d", (localtime)[1]);
    $hour = sprintf( "%02d", (localtime)[2]);

    return FormatTimeStr("$hour:$min:$sec");
}

###############################################################################
# Procedure:   FormatDateTimeStr
#
# Description: Formats a date and a time according to the configuration file,
#              and if both a date and a time are present, inserts the separator
#              between them.
#
# Parameters:  $date - The date in YYYYMMDD format
#              $time - The time in HH:MM:SS format
#
# Return vals: the formatted string
###############################################################################
sub FormatDateTimeStr {
    local($date, $time) = @_;

    if ($date ne '') {
        $date = &FormatDateStr($date);
    }
    if ($time ne '') {
        $time = &FormatTimeStr( $time );
    }

    if (($date ne '') && ($time ne '')) {
        return "$date$DATE_TIME_SEPARATOR$time";
    }
    elsif ($date ne '') {
        return $date;
    }
    elsif ($time ne '') {
        return $time;
    }
    else {
        return "";
    }
}

###############################################################################
# Procedure:   FormatDateStr
#
# Description: Formats a date according to the configuration file.
#
# Parameters:  $date - The date in YYYYMMDD format
#
# Return vals: The formated date string
###############################################################################
sub FormatDateStr {
    local($date) = @_;
    local($i, $century, $shortyear, $month, $day, $year, @format, $ch, $count, $str);

	($century, $shortyear, $month, $day) = $date =~ /(\d\d)(\d\d)(\d\d)(\d\d)/;
    $year = "$century$shortyear";

    @format = split( //, $DATE_OUTPUT_FORMAT);

    $str = "";
    for ($i=0; $i<length($DATE_OUTPUT_FORMAT); ) {
        $ch = $format[$i];
        if ($ch eq $YEAR_LETTER) {
            $count = 1;
            $i++;
            while ( $format[$i] eq $YEAR_LETTER ) {
                $count++;
                $i++;
            }
            if ($count == 2) {
                $str = "$str$shortyear";
            }
            else {
                $str = "$str$year";
            }
        }
        elsif ($ch eq $MONTH_LETTER) {
            $i++;
            while ( $format[$i] eq $MONTH_LETTER ) {
                $i++;
            }
            $str = "$str$month";
        }
        elsif ($ch eq $DAY_LETTER) {
            $i++;
            while ( $format[$i] eq $DAY_LETTER ) {
                $i++;
            }
            $str = "$str$day";
        }
        else {
            $str = "$str$ch";
            $i++;
        }
    }
    return $str;
}

###############################################################################
# Procedure:   FormatTimeStr
#
# Description: Formats a time according to the configuration file.
#
# Parameters:  $time - The time in HH:MM:SS format
#
# Return vals: The formated time string
###############################################################################
sub FormatTimeStr {
    local($time) = @_;
    local($i, $hour, $minute, $second, @format, $ch, $minute, $str);

	($hour, $minute, $second) = $time =~ /(\d\d)\:(\d\d)\:(\d\d)/;

    @format = split( //, $TIME_OUTPUT_FORMAT);

    $str = "";
    for ($i=0; $i<length($TIME_OUTPUT_FORMAT); ) {
        $ch = $format[$i];
        if ($ch eq $HOUR_LETTER) {
            $count = 1;
            $i++;
            while ( $format[$i] eq $HOUR_LETTER ) {
                $i++;
            }
            $str = "$str$hour";
        }
        elsif ($ch eq $MINUTE_LETTER) {
            $i++;
            while ( $format[$i] eq $MINUTE_LETTER ) {
                $i++;
            }
            $str = "$str$minute";
        }
        elsif ($ch eq $SECOND_LETTER) {
            $i++;
            while ( $format[$i] eq $SECOND_LETTER ) {
                $i++;
            }
            $str = "$str$second";
        }
        else {
            $str = "$str$ch";
            $i++;
        }
    }
    return $str;
}

###############################################################################
# Procedure:   FormatDaemonDateStr
#
# Description: Formats a date into a format that the daemon and app server
#              will use.
#
# Parameters:  $date - the date, assumed to be in the format specified by
#                      the configuration file
#
# Return vals: The empty string is returned if the date's format is illegal,
#              otherwise the date in YYYYMMDD format is returned
###############################################################################
sub FormatDaemonDateStr {
    local($date) = @_;
    local($year, $month, $day, @format, @date, $i, $ch);

    $year = $month = $day = "";
    if (length($DATE_INPUT_FORMAT) != length($date)) {
        return '';
    }

    @format = split( //, $DATE_INPUT_FORMAT);
    @date  = split( //, $date);

    for ($i = 0; $i < length($DATE_INPUT_FORMAT); $i++) {
        $ch = $format[$i];
        if ($ch eq $YEAR_LETTER) {
            $year = "$year$date[$i]";
        }
        elsif ($ch eq $MONTH_LETTER) {
            $month = "$month$date[$i]";
        }
        elsif ($ch eq $DAY_LETTER) {
            $day = "$day$date[$i]";
        }
        elsif ($ch eq $date[$i]) {
        }
        else {
            return '';
        }
    }
    return("$year$month$day");
}

# Needed for reuire statement in Main

1;
