#!/usr/local/bin/perl

#
# error.pl
#
# Error()
# RightViolation()
#

###############################################################################
# Procedure:   Error
#
# FUNC:        (n/a)
#
# Description: Displays an error box on the screen with the message and
#              title indicated.  Also exits the program.
#
# Parameters:  $VAR{'ERROR_MSG_VAR'} - The text of the error message.
#              $VAR{'ERROR_TITLE_VAR'} - The title of the error message.
#              $VAR{'ERROR_NAME_VAR'} - The name of the error message.
#
# Return vals: (n/a)
###############################################################################
sub Error {
    &LogItem("--> Error()");
    ($VAR{'ERROR_MSG_VAR'}, $VAR{'ERROR_TITLE_VAR'}, $VAR{'ERROR_NAME_VAR'}) = @_;
    if (!$VAR{'ERROR_MSG_VAR'}) { 
        $VAR{'ERROR_MSG_VAR'} = $UNEXPECTED_ERROR;
    }
    if (!$VAR{'ERROR_TITLE_VAR'}) { 
        $VAR{'ERROR_TITLE_VAR'} = $UNEXPECTED_ERROR_TITLE;
    }
    if (!$VAR{'ERROR_NAME_VAR'}) { 
        $VAR{'ERROR_NAME_VAR'} = $VAR{'ERROR_TITLE_VAR'};
    }

    &set_error_strings();
    &EchoFile($HTML_PATH, "error.htm");
    &LogItem("! An unrecoverable error has occured....Exiting.");
    &LogItem("<-- Error()");
    exit;
}

###############################################################################
# Procedure:   RightViolation
#
# Description: Displays an error screen and logs the problem.
#
# Parameters:  $right - an integer identifying the right that was violated.
#
# Return vals: (n/a)
###############################################################################
sub RightViolation {
    &LogItem("--> RightViolation()");
    local($right) = @_;

    &LogItem("! RightViolation: right = $right");
    &Error( $RIGHTS_VIOLATION, $RIGHTS_VIOLATION_TITLE, $RIGHTS_VIOLATION_TITLE );
    &LogItem("<-- RightViolation()");
}

###############################################################################
# Procedure:   CommunicationError
#
# Description: Displays an error screen and logs the problem.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub CommunicationError {
    &LogItem("--> CommunicationError()");
    &LogItem("! CommunicationError : system message = ($!)");
    &Error( $COMM_ERROR, $COMM_ERROR_TITLE, $COMM_ERROR_TITLE );
    &LogItem("<-- CommunicationError()");
}

###############################################################################
# Procedure:   DaemonError
#
# Description: Displays an error screen and logs the problem.
#
# Parameters:  $errCode - the integer identifier for the daemon error that occurred
#
# Return vals: (n/a)
###############################################################################
sub DaemonError {
    &LogItem("--> DaemonError()");
    local($func, $errCode) = @_;

    &LogItem("! DaemonError: $func: errCode = $errCode");
    &Error( $REQUEST_FAILED, $REQUEST_FAILED_TITLE, $REQUEST_FAILED_TITLE );
    &LogItem("<-- DaemonError()");
}

###############################################################################
# Procedure:   AppServerError
#
# Description: Displays an error screen and logs the problem.
#
# Parameters:  $errCode - the integer identifier for the App Server return value
#
# Return vals: (n/a)
###############################################################################
sub AppServerError {
    &LogItem("--> AppServerError()");
    local($func, $errCode) = @_;

    &LogItem("! AppServerError: $func: errCode = $errCode");
    &Error($REQUEST_FAILED, $REQUEST_FAILED_TITLE, $REQUEST_FAILED_TITLE);
    &LogItem("<-- AppServerError()");
}

# Needed for reuire statement in Main

1;
