#!/usr/local/bin/perl

#
# login.pl
#
# AnonymousLogin()
# CreateNewUser()
# DisplayHomepage()
# DisplayLogin()
# DisplayMenu()
# DisplayNewUser()
# DisplayTimeoutLogin()
# Entry()
# Login()
# Logout()
# StartApp()
#

###############################################################################
# Procedure:   StartApp
#
# FUNC:        (n/a)
#
# Description: Displays the initial screen the user will see.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub StartApp {
    # This file should provide the user with three options found
    # on default.htm, but for now it just displays the login prompt.
    &LogItem("--> StartApp()");
    &DisplayLogin();
    &LogItem("<-- StartApp()");
}

###############################################################################
# Procedure:   DisplayLogin
#
# FUNC:        DISP_LOGIN
#
# Description: Displays the login screen as it is with input boxes for both
#              the username and password.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayLogin {
    &LogItem("--> DisplayLogin()");

	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_GetLoginSecurity', $STRING);
		&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_NO_CALLER_CMD);
		($cmd, $subCmd, $errCode) = &RecvMsg();
		&CloseSocket();
		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
			if($argList[0] == 1) {
                # Registered users only
            }
            elsif ($argList[0] == 2) {
                # Registered & new users
                $VAR{'NEW_USER_VAR'} = $_LOGIN_NEW_USER_LINK;
            }
            elsif ($argList[0] == 3) {
                # Registered & anonymous users
                $VAR{'ANONYMOUS_LOGIN_VAR'} = $_LOGIN_ANONYMOUS_LINK;
            }
            elsif ($argList[0] == 4) {
                # Anonymous users only.
                &AnonymousLogin();
                exit;
            }
            else {
                &AppServerError( 'DisplayLogin', $argList[0] );
            }
			$done = 1;
        }
        else {
            $try++;
        }
    } continue {
		$try++;
	}

	if($done eq 0) {
        &DaemonError( 'DisplayLogin', $errCode );
	}

	$VAR{'USER_ID_VAR'} = $userID;
    $VAR{'USER_ID_INPUT_VAR'} = "<INPUT NAME=\"LOGIN_USER_ID\" SIZE=\"15\" MAXLENGTH=\"15\" VALUE=\"<<USER_ID_VAR>>\">";
    $VAR{'REMINDER_VAR'} = $_LOGIN_REMINDER_LINK;
    &set_login_strings();
	&EchoFile($HTML_PATH, "login.htm");
    &LogItem("<-- DisplayLogin()");
}

###############################################################################
# Procedure:   DisplayTimeoutLogin
#
# FUNC:        (n/a)
#
# Description: Shows the login screen when the user must reenter his/her password
#              from a daemon timeout (i.e. ValidateUser failed).  Only the password
#              box is displayed because the user ID must be the same as when the
#              user first logged in.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayTimeoutLogin {
    &LogItem("--> DisplayTimeoutLogin()");

    # Grab the parameters from the CGI and save them in this form
    # for use when the user can successfully login.
    $hidden = '';
    foreach $key (keys %FORM) {
        if ($key eq 'FUNC') {
            if ($FORM{'FUNC'} ne 'LOGIN') {
                # Store the function the user was attempting in 'LAST'.
                $hidden .= &Hidden( 'LAST', $FORM{'FUNC'} );
            }
            next;
        }

        # Skip LOGIN_PASSWORD if it exists.
        next if $key eq 'LOGIN_PASSWORD';

        # Add the key/value pair.
        $hidden .= &Hidden( $key, $FORM{$key} );
    }

    # Be sure that the LOGIN_USER_ID tag exists and is hidden in the form.
    if (!exists($FORM{'LOGIN_USER_ID'})) {
        $hidden .= &Hidden( "LOGIN_USER_ID", $userID );
    }

    $VAR{'OTHER_TAGS_VAR'} = $hidden;
    
    $VAR{'USER_ID_VAR'} = $userID;
    $VAR{'USER_ID_INPUT_VAR'} = &Bold( $userID );

    $VAR{'INFORMATION_VAR'} = &InfoStyle( "$SESSION_EXPIRED", "RED" );
    &set_login_strings();
	&EchoFile($HTML_PATH, "login.htm");
    &LogItem("<-- DisplayTimeoutLogin()");
}

###############################################################################
# Procedure:   Login
#
# FUNC:        LOGIN
#
# Description: Attempts to perform a login.  If the login is because of a session
#              timeout, it returns a 1 or a 0.  Otherwise, it displays the entry page
#              on success and redisplays the login screen on error.
#
# Parameters:  (n/a)
#
# Return vals: If it is a timeout login, 1 on success, 0 otherwise.
###############################################################################
sub Login {
    &LogItem("--> Login()");
    local($loginStatus);

    if ($FORM{'LOGIN_USER_ID'}) {
        $userID = $FORM{'LOGIN_USER_ID'};
        $password = $FORM{'LOGIN_PASSWORD'};

		$try = 0;
		$done = 0;
		while(($try < $MAX_RETRIES)&&($done eq 0)) {
		    if(!&CreateSocket($DAEMON, $PORT)) {
				next;
			}
            &AddArg($KML_FILE, $STRING);
            &AddArg('WA_Login', $STRING);
			&AddArg($userID, $STRING);
			&AddArg($password, $STRING);
			&SendMsg($CGI_TO_DAEMON, $CD_LOGIN_CMD);
			($cmd, $subCmd, $errCode) = &RecvMsg();
			&CloseSocket();
			if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_LOGIN_RSP)&&($errCode == 0)) {
				if($argList[0] == $WA_SUCCESS) {
                    # Print the content-type header so we can set the cookies.
					print "Content-type: text/html; charset=utf-8\n";

                    # Grab the return values from the daemon.
                    $userAuthID     = $argList[1];
                    $cookieTime     = $argList[2];
                    $contactID      = $argList[3];
                    $organizationID = $argList[4];
                    $createRight    = $argList[5];
                    $viewRight      = $argList[6];
                    $updateRight    = $argList[7];
                    $diagRight      = $argList[8];
                    $knowMask       = $argList[9];

                    # Set the cookies for this session.
                    &SetCompressedCookies('TPM-WA',
                        'UserID', $userID, 'UserAuthID', $userAuthID, 'CookieTime', $cookieTime,
                        'ContactID', $contactID, 'OrganizationID', $organizationID,
                        'Create', $createRight,
                        'View', $viewRight,
                        'Update', $updateRight,
                        'Diag', $diagRight,
                        'A_Create', 0, 'A_View', 0, 'A_Update', 0, 'A_Diag', 0,
                        'KnowMask', $knowMask);

                    # Print a newline to finish the headers.
					print "\n";
                    $loginStatus = 1;
				}
				elsif ($argList[0] == $WA_WRONG_PASSWORD) {
					$VAR{'INFORMATION_VAR'} = &InfoStyle( $INVALID_PASSWORD, "RED" );
                    $loginStatus = 0;
				}
                elsif ($argList[0] == $WA_INVALID_ACCESS_ID) {
                    $VAR{'INFORMATION_VAR'} = &InfoStyle( $INVALID_USER_ID, "RED" );
                    $loginStatus = 0;
                }
                else { # Unknown return value
                    &AppServerError( 'Login', $argList[0] );
                }
				$done = 1;
			}
			else {
                $try++;
			}
	    } continue {
			$try++;
		}

		if($done eq 0) {
			&DaemonError( 'Login', $errCode );
		}
	}
	else {
        # At least one of the username or password fields were blank.
		$list = "";
		if ($FORM{'LOGIN_USER_ID'} eq '') {
			$list = "$list<B>$USER_ID</B>, ";
		}

        chop $list; chop $list;  # Get rid of trailing space and comma.
        $VAR{'INFORMATION_VAR'} = &InfoStyle( "$MANDATORY_FIELDS $list", "RED" );
        $loginStatus = 0;
	}

    # Take action depending on whether this is a timeout login or a normal login
    if ($FORM{'LAST'} ne '') {  # This was a timeout login.
		if ($FORM{'LAST'} eq 'LOGOUT') {
	        if ($loginStatus) {
				DisplayHomePage();	        }
	        else {
	            &DisplayLogin();
	        }
		}
        elsif ($loginStatus) {
            return(1);
        }
        else {
            &DisplayTimeoutLogin();
		    &LogItem("<-- Login()");
            return(0);
        }
    }
    else {                      # This was a normal login.
        if ($loginStatus) {
            $VAR{'START_PAGE_VAR'} = "START_PAGE=DISP_HOMEPAGE";
            &set_entry_strings();
            &EchoFile($HTML_PATH, "entry.htm");
        }
        else {
            &DisplayLogin();
        }
    }
    &LogItem("<-- Login()");
}

###############################################################################
# Procedure:   AnonymousLogin
#
# FUNC:        ANONYMOUS_LOGIN
#
# Description: Performs an anonymous login, asking the daemon what rights the
#              user should have and displaying the entry page.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub AnonymousLogin {
    &LogItem("--> AnonymousLogin()");

	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_AnonymousLogin', $STRING);
		&SendMsg($CGI_TO_DAEMON, $CD_ANONYMOUS_LOGIN_CMD);
		($cmd, $subCmd, $errCode) = &RecvMsg();
		&CloseSocket();
		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_ANONYMOUS_LOGIN_RSP)&&($errCode == 0)) {
			if($argList[0] == $WA_SUCCESS) {
                # Print the content-type so we can set the cookies.
                print "Content-type: text/html; charset=utf-8\n";

                # Grab the appropriate arguments.
                # $argList[1] should always be 0 for anonymous logins.
                $userID         = '';
                $userAuthID     = $argList[1];
                $contactID      = $argList[2];
                $organizationID = $argList[3];
                $a_createRight  = $argList[4];
                $a_viewRight    = $argList[5];
                $a_updateRight  = $argList[6];
                $a_diagRight    = $argList[7];
                $knowMask       = $argList[8];

                # Set the cookies.
				&SetCompressedCookies('TPM-WA',
				    'UserID', $userID, 'UserAuthID', $userAuthID, 'CookieTime', 0,
                    'ContactID', $contactID, 'OrganizationID', $organizationID,
                    'Create', 0, 'View', 0, 'Update', 0, 'Diag', 0,
                    'A_Create', $a_createRight,
                    'A_View', $a_viewRight,
                    'A_Update', $a_updateRight,
                    'A_Diag', $a_diagRight,
                    'KnowMask', $knowMask);

                # Print a newline to finish the headers.
                print "\n";

                if (exists($FORM{'PROBLEM_ID'})) {
                    $VAR{'START_PAGE_VAR'} = "START_PAGE=PROB_DETAIL&PROBLEM_ID=$FORM{'PROBLEM_ID'}";
                }
                else {
                    $VAR{'START_PAGE_VAR'} = "START_PAGE=DISP_HOMEPAGE";
                }

                &set_entry_strings();
                &EchoFile($HTML_PATH, "entry.htm");
			}
            elsif ($argList[0] == $WA_LOGIN_SECURITY_BREACH) {
                &LogItem( "AnonymousLogin: security breach attempted" );
                &Error($ANONYMOUS_LOGIN_FAILED, $ANONYMOUS_LOGIN_FAILED_TITLE, 
                      $ANONYMOUS_LOGIN_FAILED_TITLE);
            }
            else { # Unknown return value
                &AppServerError( 'AnonymousLogin', $errCode );
            }
			$done = 1;
		}
		else {
			$try++;
		}
    } continue {
		$try++;
	}

	if($done eq 0) {
		&DaemonError( 'AnonymousLogin', $errCode );
	}

    &LogItem("<-- AnonymousLogin()");
}

###############################################################################
# Procedure:   Logout
#
# FUNC:        LOGOUT
#
# Description: Tells the daemon to log the user out, sets meaningless values
#              in the cookies, and displays the logout page.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub Logout {
    &LogItem("--> Logout()");
	print "Content-type: text/html; charset=utf-8\n";

    # Set the cookies.  Make UserAuthID invalid so they have to re-login
    &SetCompressedCookies('TPM-WA',
        'UserID', $userID, 'UserAuthID', -1, 'CookieTime', -1,
        'ContactID', $contactID, 'OrganizationID', $organizationID,
        'Create', $createRight, 'View', $viewRight, 'Update', $updateRight, 'Diag', $diagRight,
        'A_Create', $a_createRight, 'A_View', $a_viewRight, 'A_Update', $a_updateRight, 'A_Diag', $a_diagRight,
        'KnowMask', $knowMask);

	print "\n";
    &set_logout_strings();
	if($FORM{'TYPE'} eq 1) {
		&DisplayTimeoutLogin();
		exit;
	}
	&DisplayLogin();
    &LogItem("<-- Logout()");
}

###############################################################################
# Procedure:   Entry
#
# FUNC:        ENTRY
#
# Description: Echoes 'frameset.htm', which sets up the frames for the application.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub Entry {
    &LogItem("--> Entry()");

    if (exists($FORM{'START_PAGE'})) {
        $str = '';
        foreach $key (keys %FORM) {
            next if ($key eq 'FUNC');
            if ($key eq 'START_PAGE') {
                $str .= "FUNC=$FORM{$key}&";
                next;
            }
            $str .= "$key=$FORM{$key}&";
        }

        chop $str; # Remove trailing '&'

        $VAR{'START_PAGE_VAR'} = $str;
    }
    else {
        $VAR{'START_PAGE_VAR'} = "FUNC=DISP_HOMEPAGE";
    }

    &set_frameset_strings();
    &EchoFile($HTML_PATH, "frameset.htm");
    &LogItem("<-- Entry()");
}

###############################################################################
# Procedure:   DisplayMenu
#
# FUNC:        DISP_MENU
#
# Description: Builds the menu in the lefthand frame, based on the rights of
#              the user.  If you change this function, you should also change
#              BuildFooter() in 'misc.pl'.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayMenu {
    &LogItem("--> DisplayMenu()");
    local($str);

    $str = "";
    $str .= &Bold( &Link( $HOME_PAGE,
                        "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_HOMEPAGE",
                        "MAIN") );

    if (($diagRight != 0) || ($a_diagRight != 0)) {
        $str .= &BR() . &BR();
        $str .= &Bold( &Link( $SOLVE_PROBLEM,
                            "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_DIAGNOSTICS",
                            "MAIN") );
    }

    if (($createRight == 1) || ($a_createRight != 0)) {
        $str .= &BR() . &BR();
        $str .= &Bold( &Link( $SUBMIT_PROBLEM,
                            "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_SUBMIT",
                            "MAIN") );
    }

    # Problem inquiries are for Registered users only
    if ($viewRight > 0) {
        $str .= &BR() . &BR();
        $str .= &Bold( &Link( $SEARCH_PROBLEMS,
                            "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_PROB_INQUIRY",
                            "MAIN") );

        $str .= &BR() . &BR();
        $str .= &Bold( &Link( $VIEW_YOUR_PROBLEMS,
                            "$SCRIPT_PATH/web_access.cgi?FUNC=PROB_LIST",
                            "MAIN") );
    }

    if ($userID ne '') {
        $str .= &BR() . &BR();
        $str .= &Bold( &Link( $USER_CONFIGURATION,
                            "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_USER_CFG",
                            "MAIN") );
    }

    $str .= &BR() . &BR();
    $str .= &Bold( &Link( $HELP,
                        "$HTML_PATH/help.htm",
                        "MAIN") );

    $str .= &BR() . &BR();
    $str .= &Bold( &Link( $LOGOUT,
                        "$SCRIPT_PATH/web_access.cgi?FUNC=LOGOUT",
                        "_top") );

    $VAR{'MENU_OPTIONS_VAR'} = $str;
    &set_menu_strings();
	&EchoFile($HTML_PATH, "menu.htm");
    &LogItem("<-- DisplayMenu()");
}

###############################################################################
# Procedure:   DisplayHomePage
#
# FUNC:        DISP_HOMEPAGE
#
# Description: Displays the page that the user sees first upon login.  It varies
#              depending on what rights the user has.  If the user has view rights,
#              a request is sent to the daemon asking for the most recently
#              submitted problems for an user.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayHomePage {
    &LogItem("--> DisplayHomePage()");
	&ValidateUser();

	if ($viewRight > 0) {
        # Display the most recently submitted problems for this user.
		$try = 0;
		$done = 0;
		while(($try < $MAX_RETRIES)&&($done eq 0)) {
		    if(!&CreateSocket($DAEMON, $PORT)) {
				next;
			}
            &AddArg($KML_FILE, $STRING);
            &AddArg('WA_ProblemInquiry', $STRING);
    		&AddArg($userAuthID, $STRING);
            &AddArg('1', $STRING);           # View own problems
   		    &AddArg('', $STRING);            # Request_ID
    		&AddArg('', $STRING);            # System
	    	&AddArg('', $STRING);            # Problem code
		    &AddArg('', $STRING);            # Begin Date
    		&AddArg('', $STRING);            # End Date
            &AddArg('1', $STRING);           # Start Index
            &AddArg($MAX_MRU_PROB, $STRING); # Number Requested
    		&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_CMD);
	    	($cmd, $subCmd, $errCode) = &RecvMsg();
		    &CloseSocket();

    		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
	    		$argIdx = 1;
		    	$probList = "";
			    if($argList[0] > 0) {
    				for($i=0; $i < $argList[0]; $i++) {
	    				$probList = "$probList<TR>\n";
		    			$probList = "$probList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"30%\"><A HREF=\"$SCRIPT_PATH/web_access.cgi?FUNC=PROB_DETAIL&PROBLEM_ID=$argList[$argIdx]\">$argList[$argIdx++]</A></TD>\n"; # Problem ID
			    		$probList = "$probList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"20%\">$argList[$argIdx++]</TD>\n"; # Status
				    	$probList = "$probList<TD ALIGN=LEFT VALIGN=TOP WIDTH=\"50%\">$argList[$argIdx++]</TD>\n"; # Description
					    $probList = "$probList</TR>\n";
    				}
	    			$VAR{'PROB_LIST_VAR'} = $probList;
                    $VAR{'HOMEPAGE_VIEW_TRAILER_VAR'} = $_HOMEPAGE_VIEW_TRAILER;
                    $VAR{'VIEW_YOUR_PROBLEMS_LINK'} = &Link( $VIEW_YOUR_PROBLEMS,
                                                            "$SCRIPT_PATH/web_access.cgi?FUNC=PROB_LIST");
       			}
    			elsif ($argList == 0) {
    				$VAR{'PROB_LIST_VAR'} = &Center( &Bold( $NO_PROBLEMS_FOUND ) );
    			}
                else {
                    &AppServerError( 'DisplayHomePage', $argList[0] );
                }
                $VAR{'CURRENT_DATE_VAR'} = &CurrentShortDate();
				&SetAutoLogoutTag();
                &set_homepage_view_strings();
                &EchoFile($HTML_PATH, "homepage_view.htm");
				$done = 1;
    		}
    		else {
				$try++;
    		}
	    } continue {
			$try++;
		}
		
		if($done eq 0) {
			&DaemonError( 'DisplayHomePage', $errCode );
		}

    }
    elsif (($diagRight > 0) || ($a_diagRight > 0)) {
        # Display the diagnostic table so the user can diagnose problems.
        $VAR{'DIAGNOSTIC_LINKS_VAR'} = &GetDiagnosticTable();
        $VAR{'CURRENT_DATE_VAR'} = &CurrentShortDate();

		&SetAutoLogoutTag();
        &set_homepage_diagnose_strings();
	    &EchoFile($HTML_PATH, "homepage_diagnose.htm");
    }
    elsif (($createRight > 0) || ($a_createRight > 0)) {
        # Display a version of the submit screen.
        $VAR{'CURRENT_DATE_VAR'} = &CurrentShortDate();
        if (&GetSystemFlag()) {
            &SystemBox();
        }
        &DescriptionBox();
        if ($userID eq '') {
            $VAR{'EMAIL_BOX_VAR'} = $_EMAIL_BOX_TAG;
        }
		&SetAutoLogoutTag();
        &set_homepage_submit_strings();
        &EchoFile($HTML_PATH, "homepage_submit.htm");
    }
    else {
        &DisplayNoRights();
    }
    &LogItem("<-- DisplayHomePage()");
}

###############################################################################
# Procedure:   DisplayNewUser
#
# FUNC:        DISP_NEW_USER
#
# Description: Displays the new user registration screen.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayNewUser {
    &LogItem("--> DisplayNewUser()");
    $VAR{'USER_ID_VAR'} = $FORM{'USER_ID'};
    $VAR{'EMAIL_ADDRESS_VAR'} = $FORM{'EMAIL_ADDRESS'};
    $VAR{'FIRST_NAME_VAR'} = $FORM{'FIRST_NAME'};
    $VAR{'MIDDLE_NAME_VAR'} = $FORM{'MIDDLE_NAME'};
    $VAR{'LAST_NAME_VAR'} = $FORM{'LAST_NAME'};
    $VAR{'PHONE_NUMBER_VAR'} = $FORM{'PHONE_NUMBER'};

    &set_newuser_strings();
   	&EchoFile($HTML_PATH, "newuser.htm");
    &LogItem("<-- DisplayNewUser()");
}

###############################################################################
# Procedure:   CreateNewUser
#
# FUNC:        CREATE_NEW_USER
#
# Description: Attempts to create new records for the user in the database.
#              Displays the login screen on success, and redisplays the new
#              user screen on failure.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub CreateNewUser {
    &LogItem("--> CreateNewUser()");
    if (($FORM{'USER_ID'})&&($FORM{'LAST_NAME'})&&($FORM{'EMAIL_ADDRESS'})) {
		$try = 0;
		$done = 0;
		while(($try < $MAX_RETRIES)&&($done eq 0)) {
		    if(!&CreateSocket($DAEMON, $PORT)) {
				next;
			}
            &AddArg($KML_FILE, $STRING);
            &AddArg('WA_CreateNewUser', $STRING);
			&AddArg($FORM{'USER_ID'}, $STRING);
			&AddArg($FORM{'EMAIL_ADDRESS'}, $STRING);
            &AddArg($FORM{'FIRST_NAME'}, $STRING);
            &AddArg($FORM{'MIDDLE_NAME'}, $STRING);
            &AddArg($FORM{'LAST_NAME'}, $STRING);
            &AddArg($FORM{'PHONE_NUMBER'}, $STRING);
			&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_NO_CALLER_CMD);
			($cmd, $subCmd, $errCode) = &RecvMsg();
			&CloseSocket();
			if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
				if($argList[0] == $WA_SUCCESS) {
                    $userID = $FORM{'USER_ID'};
                    $VAR{'INFORMATION_VAR'} = &InfoStyle( $NEW_USER_REG_SUCCESS, "BLUE" );
                    &DisplayLogin();
				}
				elsif ($argList[0] == $WA_ACCESS_ID_EXISTS) {
                    $VAR{'INFORMATION_VAR'} = &InfoStyle( $ACCESS_ID_EXISTS, "RED" );
                    &DisplayNewUser();
                }
 				elsif ($argList[0] == $WA_UNIQUE_NAME_FAILED) {
                    $VAR{'INFORMATION_VAR'} = &InfoStyle( $UNIQUE_NAME_FAILED, "RED" );
                    &DisplayNewUser();
                }
                else { # Unknown return value
                    &AppServerError( 'CreateNewUser', $argList[0] );
                }
				$done = 1;
			}
			else {
				$try++;
			}
	    } continue {
			$try++;
		}
		
		if($done eq 0) {
	        &DaemonError( 'CreateNewUser', $errCode );
		}
	}
	else {
		$list = "";
		if ($FORM{'USER_ID'} eq '') {
			$list = "$list<B>$USER_ID</B>, ";
		}
		if ($FORM{'LAST_NAME'} eq '') {
			$list = "$list<B>$LAST_NAME</B>, ";
		}
        if ($FORM{'EMAIL_ADDRESS'} eq '') {
			$list = "$list<B>$EMAIL_ADDRESS</B>, ";
		}
        chop $list; chop $list;  # Get rid of trailing space and comma.
        $VAR{'INFORMATION_VAR'} = &InfoStyle( "$MANDATORY_FIELDS $list", "RED" );
        &DisplayNewUser();
	}
    &LogItem("<-- CreateNewUser()");
}

###############################################################################
# Procedure:   DisplayReminder
#
# FUNC:        DISP_REMINDER
#
# Description: Displays the user ID and password reminder screen.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayReminder {
    &LogItem("--> DisplayReminder()");
    $VAR{'EMAIL_ADDRESS_VAR'} = $FORM{'EMAIL_ADDRESS'};

    &set_reminder_strings();
   	&EchoFile($HTML_PATH, "reminder.htm");
    &LogItem("<-- DisplayReminder()");
}

###############################################################################
# Procedure:   Reminder
#
# FUNC:        REMINDER
#
# Description: Attempts to send a notification to the user telling the user   
#              his/her user ID and password.  It suceeds if the user has the
#              appropriate notification type (email address) setup.  If it fails,
#              it returns to the Reminder screen.  If it succeeds, it returns to
#              the main login screen.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub Reminder {
    &LogItem("--> Reminder()");
    if ($FORM{'EMAIL_ADDRESS'}) {
		$try = 0;
		$done = 0;
		while(($try < $MAX_RETRIES)&&($done eq 0)) {
		    if(!&CreateSocket($DAEMON, $PORT)) {
				next;
			}
            &AddArg($KML_FILE, $STRING);
            &AddArg('WA_SendLoginInfo', $STRING);
			&AddArg($FORM{'EMAIL_ADDRESS'}, $STRING);
			&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_NO_CALLER_CMD);
			($cmd, $subCmd, $errCode) = &RecvMsg();
			&CloseSocket();
			if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
				if($argList[0] == $WA_SUCCESS) {
                    $VAR{'INFORMATION_VAR'} = &InfoStyle( $REMINDER_SUCCESS, "BLUE" );
                    &DisplayLogin();
				}
				elsif ($argList[0] == $WA_INVALID_EMAIL_ADDRESS) {
                    $VAR{'INFORMATION_VAR'} = &InfoStyle( $REMINDER_INVALID_EMAIL, "RED" );
                    &DisplayReminder();
                }
                else { # Unknown return value
                    &AppServerError( 'Reminder', $argList[0] );
                }
				$done = 1;
			}
			else {
				$try++;
			}
	    } continue {
			$try++;
		}

		if($done eq 0) {
	        &DaemonError( 'Reminder', $errCode );
		}
	}
	else {
        $VAR{'INFORMATION_VAR'} = &InfoStyle( "$MANDATORY_FIELDS <B>$EMAIL_ADDRESS</B>", "RED" );
        &DisplayReminder();
	}
    &LogItem("<-- Reminder()");
}

# Needed for reuire statement in Main

1;
