#!/usr/local/bin/perl

#
# misc.pl
#
# BuildFooter()
# DescriptionBox()
# DisplayNoRights()
# GetDiagnosticTable()
# GetProbCodeList()
# GetSystemFlag()
# GetSystemList()
# PrevNextButtons()
# SystemBox()
# ValidateUser()
# VerifyDiagRight()
# VerifyUpdateRight()
# VerifyViewRight()
# SetAutoLogoutTag()
#

###############################################################################
# Procedure:   ValidateUser
#
# Description: Asks the daemon to validate the authorization ID for security.
#              If the user is not validated, the timeout login screen is shown.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub ValidateUser {
    &LogItem("--> ValidateUser()");
	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($cookieTime, $INTEGER);
		&SendMsg($CGI_TO_DAEMON, $CD_VALID_USER_CMD);
		($cmd, $subCmd, $errCode) = &RecvMsg();
		&CloseSocket();
		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_VALID_USER_RSP)&&($errCode == 0)) {
			if($argList[0]) {

                # Validation okay. Update Cookie Time, so that we do not time out.
				$cookieTime =  $argList[1];

                # Print the content-type header so we can set the cookies.
				if(!$CookieSet) {
					print "Content-type: text/html; charset=utf-8\n";
	                &SetCompressedCookies('TPM-WA',
	                    'UserID', $userID, 'UserAuthID', $userAuthID, 'CookieTime', $cookieTime,
	                    'ContactID', $contactID, 'OrganizationID', $organizationID,
	                    'Create', $createRight,
	                    'View', $viewRight,
	                    'Update', $updateRight,
	                    'Diag', $diagRight,
	                    'A_Create', $a_createRight,
	                    'A_View', $a_viewRight,
	                    'A_Update', $a_updateRight,
	                    'A_Diag', $a_diagRight,
	                    'KnowMask', $knowMask);

	                # Print a newline to finish the headers.
					print "\n";
				}
				return(1);
			}
			else {
                # User is not validated.  Let the user login again.
                &DisplayTimeoutLogin();
                exit;
			}
			$done = 1;
		}
		else {
			$try++;
		}
    } continue {
		$try++;
	}

	if($done eq 0) {
	    &DaemonError( 'ValidateUser', $errCode );
	}

    &LogItem("<-- ValidateUser()");
}

###############################################################################
# Procedure:   VerifyDiagRight
#
# Description: Checks $FORM{'DIAGNOSTIC_TYPE'} and the $diagRight variable to
#              see if the user has rights to use the diagnostic type being
#              attempted.
#
# Parameters:  (n/a)
#
# Return vals: nonzero if the operation is allowed, 0 otherwise.
###############################################################################
sub VerifyDiagRight {
    &LogItem("--> VerifyDiagRight()");
    if ($userID ne '') {
        $right = $diagRight;
    }
    else {
        $right = $a_diagRight;
    }

    $diag = $FORM{'DIAGNOSTIC_TYPE'};

    if ($diag == 1) {
        &LogItem("<-- VerifyDiagRight():1");
        return ($right & $ADAPTIVE_LEARNING_BIT);
    }
    elsif ($diag == 2) {
        &LogItem("<-- VerifyDiagRight():2");
        return ($right & $COMMON_PROBLEMS_BIT);
    }
    elsif ($diag == 3) {
        &LogItem("<-- VerifyDiagRight():3");
        return ($right & $HOT_NEWS_BIT);
    }
    elsif ($diag == 4) {
        &LogItem("<-- VerifyDiagRight():4");
        return ($right & $ERROR_MESSAGES_BIT);
    }
    elsif ($diag == 5) {
        &LogItem("<-- VerifyDiagRight():5");
        return ($right & $HYPER_TREES_BIT);
    }

    &LogItem("<-- VerifyDiagRight():0");
    return 0;
}

###############################################################################
# Procedure:   VerifyViewRight
#
# Description: Checks $VAR{'CONTACT_VAR'} and $VAR{'ORGANIZATION_VAR'} along
#              with $contactID and $organizationID and $viewRight to check whether
#              the user has permission to view the problem based on this info.
#
# Parameters:  (n/a)
#
# Return vals: nonzero if the operation is allowed, 0 otherwise.
###############################################################################
sub VerifyViewRight {
    &LogItem("--> VerifyViewRight()");
    if ($userID ne '') {
        $right = $viewRight;
    }
    else {
        $right = $a_viewRight;
    }

    if ($right == 0) {    # View none
        &LogItem("<-- VerifyViewRight():0");
        return 0;
    }
    elsif ($right == 1) { # View own
        &LogItem("<-- VerifyViewRight():1");
        return ($VAR{'CONTACT_VAR'} eq $contactID);
    }
    elsif ($right == 2) { # View own + default organization's
        &LogItem("<-- VerifyViewRight():2");
        return (($VAR{'CONTACT_VAR'} eq $contactID) ||
                ($problemOrganizationID eq $organizationID));
    }
    elsif ($right == 3) { # View all
        &LogItem("<-- VerifyViewRight():3");
        return 1;
    }

    &LogItem("<-- VerifyViewRight()");
    return 0;
}

###############################################################################
# Procedure:   VerifyUpdateRight
#
# Description: Checks $VAR{'CONTACT_VAR'} and $VAR{'ORGANIZATION_VAR'} along
#              with $contactID and $organizationID and $updateRight to check whether
#              the user has permission to view the problem based on this info.
#
# Parameters:  (n/a)
#
# Return vals: nonzero if the operation is allowed, 0 otherwise.
###############################################################################
sub VerifyUpdateRight {
    &LogItem("--> VerifyUpdateRight()");
    if ($userID ne '') {
        $right = $updateRight;
    }
    else {
        $right = $a_updateRight;
    }

    if ($right == 0) {    # Update none
        &LogItem("<-- VerifyUpdateRight():0");
        return 0;
    }
    elsif ($right == 1) { # Update own
        &LogItem("<-- VerifyUpdateRight():1");
        return ($VAR{'CONTACT_VAR'} eq $contactID);
    }
    elsif ($right == 2) { # Update own + default organization's
        &LogItem("<-- VerifyUpdateRight():2");
        return (($VAR{'CONTACT_VAR'} eq $contactID) ||
                ($problemOrganizationID eq $organizationID));
    }
    elsif ($right == 3) { # Update all
        &LogItem("<-- VerifyUpdateRight():3");
        return 1;
    }

    &LogItem("<-- VerifyUpdateRight()");
    return 0;
}

###############################################################################
# Procedure:   BuildFooter
#
# Description: Assigns $VAR{'COMMON_FOOTER_VAR'} to a list of the options
#              the user has.  When changing this function, also change DisplayMenu
#              in 'login.pl'.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub BuildFooter {
    &LogItem("--> BuildFooter()");
    local($str);

    $str = "<FONT SIZE=\"-1\">";
    $str .= '[' . &Link( $HOME_PAGE,
                        "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_HOMEPAGE" ) .
            '] ';

    if (($diagRight != 0) || ($a_diagRight != 0)) {
        $str .= '[' . &Link( $SOLVE_PROBLEM,
                            "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_DIAGNOSTICS" ) .
                '] ';
    }

    if (($createRight == 1) || ($a_createRight == 1)) {
        $str .= '[' . &Link( $SUBMIT_PROBLEM,
                            "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_SUBMIT" ) .
                '] ';
    }

    # Only allow inquiries for registered users!
    if ($viewRight > 0) {
        $str .= '[' . &Link( $SEARCH_PROBLEMS,
                            "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_PROB_INQUIRY" ) .
                '] ';

        $str .= '[' . &Link( $VIEW_YOUR_PROBLEMS,
                            "$SCRIPT_PATH/web_access.cgi?FUNC=PROB_LIST" ) .
                '] ';
    }

    if ($userID ne '') {
        $str .= '[' . &Link( $USER_CONFIGURATION,
                            "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_USER_CFG" ) .
                '] ';
    }

    $str .= '[' . &Link( $HELP,
                        "$HTML_PATH/help.htm" ) .
            '] ';


    $str .= '[' . &Link( $LOGOUT,
                        "$SCRIPT_PATH/web_access.cgi?FUNC=LOGOUT",
                        "_top") .
            '] ';


    $str .= "</FONT>";

    $VAR{'COMMON_FOOTER_VAR'} = $str;
    &LogItem("<-- BuildFooter()");
}

###############################################################################
# Procedure:   GetDiagnosticTable
#
# Description: Builds the table of links and explanations of the diagnostic
#              tools the user has rights to use.
#
# Parameters:  (n/a)
#
# Return vals: the HTML code for the table
###############################################################################
sub GetDiagnosticTable {
    &LogItem("--> GetDiagnosticTable()");
    if ($userID ne '') {
        $right = $diagRight;
    }
    else {
        $right = $a_diagRight;
    }

    local($str);

    $str = "";

    if ($right & $ADAPTIVE_LEARNING_BIT) {
        $str .=
            &TR( &TD( &Link( $ADAPTIVE_LEARNING,
                          "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_DIAG_INPUT&DIAGNOSTIC_TYPE=1"
                        ) ) .
                &TD( $_ADAPTIVE_LEARNING_EXPLANATION ) );
    }

    if ($right & $COMMON_PROBLEMS_BIT) {
        $str .=
            &TR( &TD( &Link( $COMMON_PROBLEMS,
                          "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_DIAG_INPUT&DIAGNOSTIC_TYPE=2"
                        ) ) .
                &TD( $_COMMON_PROBLEMS_EXPLANATION ) );
    }

    if ($right & $HOT_NEWS_BIT) {
        $str .=
            &TR( &TD( &Link( $HOT_NEWS,
                          "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_DIAG_INPUT&DIAGNOSTIC_TYPE=3"
                        ) ) .
                &TD( $_HOT_NEWS_EXPLANATION ) );
    }

    if ($right & $ERROR_MESSAGES_BIT) {
        $str .= 
            &TR( &TD( &Link( $ERROR_MESSAGES,
                          "$SCRIPT_PATH/web_access.cgi?FUNC=DISP_DIAG_INPUT&DIAGNOSTIC_TYPE=4"
                        ) ) .
                &TD( $_ERROR_MESSAGES_EXPLANATION ) );
    }

    if ($right & $HYPER_TREES_BIT) {
        $str .=
            &TR( &TD( &Link( $HYPER_TREES,
                          "$SCRIPT_PATH/web_access.cgi?FUNC=DIAGNOSE&DIAGNOSTIC_TYPE=5"
                        ) ) .
                &TD( $_HYPER_TREES_EXPLANATION ) );
    }

    &LogItem("<-- GetDiagnosticTable()");
    return ($str);
}

###############################################################################
# Procedure:   PrevNextButtons
#
# Description: Builds HTML code for the hidden tags needed when lists of problems
#              or solutions are made, but not all are displayed.
#
# Parameters:  $startIdx   - the index of the first problem to be displayed
#              $prevButton - 1 if a 'Previous' button should be displayed, 0 otherwise
#              $nextButton - 1 if a 'Next' button should be displayed, 0 otherwise
#
# Return vals: The HTML code for the buttons
###############################################################################
sub PrevNextButtons {
    &LogItem("--> PrevNextButtons()");
	local($startIdx, $prevButton, $nextButton) = @_;

	$buttonSet = "";

	if($nextButton && $prevButton) {
		$buttonSet = "$buttonSet<TABLE>\n";
		$buttonSet = "$buttonSet<TR><TD>\n";
	}

	if($prevButton) {
	    $prevStartIdx = ($startIdx - $MAX_DIAG);
		$buttonSet = "$buttonSet<FORM METHOD=\"POST\" ACTION=\"$SCRIPT_PATH/web_access.cgi\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"FUNC\" VALUE=\"$FORM{'FUNC'}\">\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"SYSTEM\" VALUE=\"$FORM{'SYSTEM'}\">\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"DESCRIPTION\" VALUE=\"$FORM{'DESCRIPTION'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"PARENT_ID\" VALUE=\"$FORM{'PARENT_ID'}\">\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"DIAGNOSTIC_TYPE\" VALUE=\"$FORM{'DIAGNOSTIC_TYPE'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"REQUEST_ID\" VALUE=\"$FORM{'REQUEST_ID'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"PROBLEM_CODE\" VALUE=\"$FORM{'PROBLEM_CODE'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"BEGIN_DATE\" VALUE=\"$FORM{'BEGIN_DATE'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"END_DATE\" VALUE=\"$FORM{'END_DATE'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"INCLUDE_ORG\" VALUE=\"$FORM{'INCLUDE_ORG'}\">\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"START_IDX\" VALUE=\"$prevStartIdx\">\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"PAGE_NUMBER\" VALUE=\"$VAR{'PAGE_NUMBER_VAR'}\">\n";
		$buttonSet = "$buttonSet<TABLE BORDER=0>\n";
		$buttonSet = "$buttonSet<TR><TD ALIGN=LEFT>\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"SUBMIT\" NAME=\"PREV\" VALUE=\"$PREVIOUS\">\n";
		$buttonSet = "$buttonSet</TD></TR>\n";
		$buttonSet = "$buttonSet</TABLE>\n";
		$buttonSet = "$buttonSet</FORM>\n";
	}

	if($nextButton && $prevButton) {
		$buttonSet = "$buttonSet</TD>\n";
		$buttonSet = "$buttonSet<TD>\n";
	}

	if($nextButton) {
	    $nextStartIdx = ($startIdx + $MAX_DIAG);
		$buttonSet = "$buttonSet<FORM METHOD=\"POST\" ACTION=\"$SCRIPT_PATH/web_access.cgi\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"FUNC\" VALUE=\"$FORM{'FUNC'}\">\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"SYSTEM\" VALUE=\"$FORM{'SYSTEM'}\">\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"DESCRIPTION\" VALUE=\"$FORM{'DESCRIPTION'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"PARENT_ID\" VALUE=\"$FORM{'PARENT_ID'}\">\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"DIAGNOSTIC_TYPE\" VALUE=\"$FORM{'DIAGNOSTIC_TYPE'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"REQUEST_ID\" VALUE=\"$FORM{'REQUEST_ID'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"PROBLEM_CODE\" VALUE=\"$FORM{'PROBLEM_CODE'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"BEGIN_DATE\" VALUE=\"$FORM{'BEGIN_DATE'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"END_DATE\" VALUE=\"$FORM{'END_DATE'}\">\n";
        $buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"INCLUDE_ORG\" VALUE=\"$FORM{'INCLUDE_ORG'}\">\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"START_IDX\" VALUE=\"$nextStartIdx\">\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"HIDDEN\" NAME=\"PAGE_NUMBER\" VALUE=\"$VAR{'PAGE_NUMBER_VAR'}\">\n";
		$buttonSet = "$buttonSet<TABLE BORDER=0>\n";
		$buttonSet = "$buttonSet<TR><TD ALIGN=LEFT>\n";
		$buttonSet = "$buttonSet<INPUT TYPE=\"SUBMIT\" NAME=\"NEXT\" VALUE=\"$NEXT\">\n";
		$buttonSet = "$buttonSet</TD></TR>\n";
		$buttonSet = "$buttonSet</TABLE>\n";
		$buttonSet = "$buttonSet</FORM>\n";
	}

	if($nextButton && $prevButton) {
		$buttonSet = "$buttonSet</TD></TR>\n";
		$buttonSet = "$buttonSet</TABLE>\n";
	}

    &LogItem("<-- PrevNextButtons()");
	return($buttonSet);
}

###############################################################################
# Procedure:   DisplayHelp
#
# FUNC:        DISP_HELP
#
# Description: Displays the help page.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayNoRights {
    &LogItem("--> DisplayNoRights()");
    $VAR{'CURRENT_DATE_VAR'} = &CurrentShortDate();
	&SetAutoLogoutTag();
	&set_homepage_no_rights_strings();
	&EchoFile($HTML_PATH, "homepage_norights.htm");
    &LogItem("<-- DisplayNoRights()");
}

###############################################################################
# Procedure:   GetSystemList
#
# Description: Asks the daemon for a list of systems in the database.  Assigns
#              the system list to $VAR{'SYSTEM_LIST_VAR'}
#
# Parameters:  (n/a)
#
# Return vals: 1 on success, 0 otherwise
###############################################################################
sub GetSystemList {
    &LogItem("--> GetSystemList()");
	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_GetSystems', $STRING);
        &AddArg($userAuthID, $STRING);
        &AddArg($knowMask, $STRING);
        &AddArg($organizationID, $STRING);
	    &SendMsg($CGI_TO_DAEMON, $CD_GET_SYSTEMS_CMD);
	    ($cmd, $subCmd, $errCode) = &RecvMsg();
	    &CloseSocket();
	    if(($cmd == $DAEMON_TO_CGI)&&(($subCmd == $DC_GET_SYSTEMS_RSP)||($subCmd == $DC_APPSERVER_FUNCTION_RSP))&&($errCode == 0)) {
	        $argIdx = 1;
	        $sysList = "";
            if ($argList[0] >= 0) {
    	        for ($i = 0; $i < $argList[0]; $i++) {
                    # If the user has selected a system, keep it selected.
                    if ($argList[$argIdx] eq $FORM{'SYSTEM'}) {
	                    $sysList = "$sysList<OPTION SELECTED VALUE=\"$argList[$argIdx]\">$argList[$argIdx++]\n";
                    }
                    else {
                        $sysList = "$sysList<OPTION VALUE=\"$argList[$argIdx]\">$argList[$argIdx++]\n";
                    }
                }
                $VAR{'SYSTEM_LIST_VAR'} = $sysList;
                &LogItem("<-- GetSystemList()");
                return(1);
            }
            else {
                &AppServerError( 'GetSystemList', $argList[0] );
	        }
			$done = 1;
	    }
	    else {
			$try++;
		}
    } continue {
		$try++;
	}
	
	if($done eq 0) {
		&DaemonError( 'GetSystemList', $errCode );
	}

    &LogItem("<-- GetSystemList():FAILED");
	return(0);
}

###############################################################################
# Procedure:   GetProbCodeList
#
# Description: Asks the daemon for a list of problem codes in the database.  Assigns
#              the system list to $VAR{'PROB_CODE_LIST_VAR'}
#
# Parameters:  (n/a)
#
# Return vals: 1 on success, 0 otherwise
###############################################################################
sub GetProbCodeList {
    &LogItem("--> GetProbCodeList()");
	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_GetProblemCodes', $STRING);
        &AddArg($userAuthID, $STRING);
        &SendMsg($CGI_TO_DAEMON, $CD_GET_PROBLEM_CODES_CMD);
        ($cmd, $subCmd, $errCode) = &RecvMsg();
        &CloseSocket();
        if(($cmd == $DAEMON_TO_CGI)&&(($subCmd == $DC_GET_PROBLEM_CODES_RSP)||($subCmd == $DC_APPSERVER_FUNCTION_RSP))&&($errCode == 0)) {
            $argIdx = 1;
            $probCodesList = "";
            if ($argList[0] >= 0) {
                for ($i = 0; $i < $argList[0]; $i++) {
                    if ($argList[$argIdx] eq $FORM{'PROBLEM_CODE'}) {
                        $probCodesList = "$probCodesList<OPTION SELECTED VALUE=\"$argList[$argIdx]\">$argList[$argIdx++]\n";
                    }
                    else {
                        $probCodesList = "$probCodesList<OPTION VALUE=\"$argList[$argIdx]\">$argList[$argIdx++]\n";
                    }
                }
                $VAR{'PROB_CODE_LIST_VAR'} = $probCodesList;
                &LogItem("<-- GetProbCodeList()");
    			return(1);
            }
            else {
                &AppServerError( 'GetProbCodeList', $argList[0] );
            }
			$done = 1;
        }
        else {
			$try++;
		}
    } continue {
		$try++;
	}

    if($done eq 0) {
		&DaemonError( 'GetProbCodeList', $errCode );
    }
    &LogItem("<-- GetProbCodeList():FAILED");
	return(0);
}

###############################################################################
# Procedure:   SystemBox
#
# Description: Assigns $VAR{'SYSTEM_BOX_VAR'} the HTML coe for a system box.
#              Uses GetSystemList() to select the system the user has selected,
#              if one has been selected.
#
# Parameters:  (n/a)
#
# Return vals: 1 on success, 0 otherwise
###############################################################################
sub SystemBox {
    &LogItem("--> SystemBox()");
    if (&GetSystemList()) {
        $VAR{'SYSTEM_BOX_VAR'} = <<EOT;
<P><FONT COLOR="BLUE"><B>$_SYSTEM_HEADING</B></FONT>
<TABLE BORDER="0">
	<TR><TD><B>$SYSTEM$_STANDARD_SEPARATOR</B></TD>
	<TD><SELECT NAME="SYSTEM">
	<OPTION VALUE="">$_SYSTEM_BOX_DEFAULT_VALUE</OPTION>
    <<SYSTEM_LIST_VAR>>
	</SELECT></TD>
	</TR>
</TABLE>
EOT
        &LogItem("<-- SystemBox()");
        return(1);
    }
    &LogItem("<-- SystemBox():FAILED");
    return (0);
}

###############################################################################
# Procedure:   DescriptionBox
#
# Description: Assigns $VAR{'DESCRIPTION_BOX_VAR'} the HTML code for a description box.
#              Puts $FORM{'DESCRIPTION'} in for the default value, so if an
#              user has typed something already, it remains.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DescriptionBox {
    &LogItem("--> DescriptionBox()");
    $VAR{'DESCRIPTION_BOX_VAR'} = <<EOT;
<BR>

<P><FONT COLOR="RED"><B>$_DESCRIPTION_HEADING</B></FONT>    
<TABLE BORDER="0">
	<TR><TD><B>$DESCRIPTION$_STANDARD_SEPARATOR$_MANDATORY_CHARACTER</B></TD>
	<TD><TEXTAREA NAME="DESCRIPTION" ROWS="4" COLS="60" WRAP=VIRTUAL>$FORM{'DESCRIPTION'}</TEXTAREA>
	</TD></TR>
</TABLE>
EOT
    &LogItem("<-- DescriptionBox()");
}

###############################################################################
# Procedure:   GetSystemFlag
#
# Description: Checks the database to see if the user should see the SystemBox
#              when submitting problems or not.
#
# Parameters:  (n/a)
#
# Return vals: 1 if the flag is 'TRUE', 0 otherwise.
###############################################################################
sub GetSystemFlag {
    &LogItem("--> GetSystemFlag()");
	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_GetSystemFlag', $STRING);
        &AddArg($userAuthID, $STRING);
		&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_CMD);
		($cmd, $subCmd, $errCode) = &RecvMsg();
		&CloseSocket();
		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
			if($argList[0] eq 'TRUE') {
                &LogItem("<-- GetSystemFlag()");
				return(1);
			}
			else {
                &LogItem("<-- GetSystemFlag()");
                return(0);
			}
			$done = 1;
		}
		else {
			$try++;
		}
    } continue {
		$try++;
	}

	if($done eq 0) {
        &DaemonError( 'GetSystemFlag', $errCode );
	}
    &LogItem("<-- GetSystemFlag():FAILED");
}

###############################################################################
# Procedure:   SetAutoLogoutTag
#
# Description: This will set the Refresh time to auto logout a user.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub SetAutoLogoutTag {
	if($userAuthID ne 0) { # User is not Anonymous
	    $VAR{'META_TAGS_VAR'} = $_REFRESH_TAG;
	}
}

# Needed for reuire statement in Main

1;
