#!/usr/local/bin/perl

#
# modify.pl
#
# ChangePassword()
# DisplayCachedModifyUser()
# DisplayModifyUser()
# ModifyUser()
#

###############################################################################
# Procedure:   DisplayModifyUser
#
# FUNC:        DISP_MODIFY_USER
#
# Description: Asks the daemon for detailed information about the user, then
#              displays a form allowing the user to edit that information, as
#              well as change his/her password.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayModifyUser {
    &LogItem("--> DisplayModifyUser");
	&ValidateUser();
	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_GetCallerInfo', $STRING);
		&AddArg($userAuthID, $STRING);
		&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_CMD);
		($cmd, $subCmd, $errCode) = &RecvMsg();
		&CloseSocket();
		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
			if($argList[0] == $WA_SUCCESS) {
				$argIdx = 1;
                $VAR{'EMAIL_ADDRESS_VAR'} = $argList[$argIdx++];
				$VAR{'FIRST_NAME_VAR'} = $argList[$argIdx++];
				$VAR{'MIDDLE_NAME_VAR'} = $argList[$argIdx++];
				$VAR{'LAST_NAME_VAR'} = $argList[$argIdx++];
				$VAR{'PHONE_NUMBER_VAR'} = $argList[$argIdx++];
                $VAR{'USER_ID_VAR'} = $userID;
				&SetAutoLogoutTag();
                &set_user_cfg_strings();
            	&EchoFile($HTML_PATH, "user_cfg.htm");
			}
			else {
                &AppServerError( 'DisplayModifyUser', $argList[0] );
			}
			$done = 1;
		}
		else {
			$try++;
		}
    } continue {
		$try++;
	}
	
	if($done eq 0) {
		&DaemonError( 'DisplayModifyUser', $errCode );
	}
    &LogItem("<-- DisplayModifyUser");
}

###############################################################################
# Procedure:   DisplayCachedModifyUser
#
# FUNC:        (n/a)
#
# Description: Displays the modify user screen without looking in the database
#              for user information because we have already gotten it.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayCachedModifyUser {
    &LogItem("--> DisplayCachedModifyUser");
    # No need to validate user, because the user has already been validated.

    $VAR{'EMAIL_ADDRESS_VAR'} = $FORM{'EMAIL_ADDRESS'};
	$VAR{'FIRST_NAME_VAR'} = $FORM{'FIRST_NAME'};
	$VAR{'MIDDLE_NAME_VAR'} = $FORM{'MIDDLE_NAME'};
	$VAR{'LAST_NAME_VAR'} = $FORM{'LAST_NAME'};
	$VAR{'PHONE_NUMBER_VAR'} = $FORM{'PHONE_NUMBER'};
    $VAR{'USER_ID_VAR'} = $userID;
	&SetAutoLogoutTag();
    &set_user_cfg_strings();
    &EchoFile($HTML_PATH, "user_cfg.htm");
    &LogItem("<-- DisplayCachedModifyUser");
}

###############################################################################
# Procedure:   ModifyUser
#
# FUNC:        MODIFY_USER
#
# Description: Asks the daemon to save the new information to the database.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub ModifyUser {
    &LogItem("--> ModifyUser");
	&ValidateUser();

    if (!($FORM{'LAST_NAME'})) {
        # Last name is required in the database!
        $list = &Bold( $LAST_NAME );
        $VAR{'INFORMATION_VAR'} = &InfoStyle( "$MANDATORY_FIELDS $list", "RED" );
        &DisplayCachedModifyUser();
        &LogItem("<-- ModifyUser");
        exit;
    }

	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_ModifyUser', $STRING);
		&AddArg($userAuthID, $STRING);
        &AddArg($FORM{'EMAIL_ADDRESS'}, $STRING);
        &AddArg($FORM{'FIRST_NAME'}, $STRING);
        &AddArg($FORM{'MIDDLE_NAME'}, $STRING);
        &AddArg($FORM{'LAST_NAME'}, $STRING);
        &AddArg($FORM{'PHONE_NUMBER'}, $STRING);
		&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_CMD);
		($cmd, $subCmd, $errCode) = &RecvMsg();
		&CloseSocket();
		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
            if ($argList[0] == $WA_SUCCESS) {
                $VAR{'INFORMATION_VAR'} = &InfoStyle( $USER_MODIFY_SUCCESS, "BLUE" );
                &DisplayModifyUser();
            }
			elsif ($argList[0] == $WA_UNIQUE_NAME_FAILED) {
                $VAR{'INFORMATION_VAR'} = &InfoStyle( $UNIQUE_NAME_FAILED, "RED" );
                &DisplayCachedModifyUser();
            }
            else {
                &AppServerError( 'ModifyUser', $argList[0] );
            }
			$done = 1;
        }
		else {
			$try++;
		}
    } continue {
		$try++;
	}

	if($done eq 0) {
		&DaemonError( 'ModifyUser', $errCode );
	}
    &LogItem("<-- ModifyUser");
}

###############################################################################
# Procedure:   ChangePassword
#
# FUNC:        CHANGE_PASSWORD
#
# Description: Asks the daemon to modify the password of the user.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub ChangePassword {
    &LogItem("--> ChangePassword");
    &ValidateUser();

    if (($FORM{'OLD_PASSWORD'} ne '')&&($FORM{'NEW_PASSWORD'} ne '')&&($FORM{'CONFIRM_PASSWORD'} ne '')) {
        $oldPass = $FORM{'OLD_PASSWORD'};
	    $newPass = $FORM{'NEW_PASSWORD'};
    	$confirmPass = $FORM{'CONFIRM_PASSWORD'};

	    if($newPass ne $confirmPass) {
            $VAR{'INFORMATION_VAR'} = &InfoStyle( $PASSWORD_CONFIRM, "RED" );
            &DisplayCachedModifyUser();
            &LogItem("<-- ChangePassword");
            exit;
    	}

    	if(length($newPass) < 5) {
            $VAR{'INFORMATION_VAR'} = &InfoStyle( $PASSWORD_SHORT, "RED" );
            &DisplayCachedModifyUser();
            &LogItem("<-- ChangePassword");
            exit;
    	}

		$try = 0;
		$done = 0;
		while(($try < $MAX_RETRIES)&&($done eq 0)) {
		    if(!&CreateSocket($DAEMON, $PORT)) {
				next;
			}
            &AddArg($KML_FILE, $STRING);
            &AddArg('WA_ChangePassword', $STRING);
    		&AddArg($userAuthID, $STRING);
            &AddArg($oldPass, $STRING);
            &AddArg($newPass, $STRING);
    		&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_CMD);
    		($cmd, $subCmd, $errCode) = &RecvMsg();
    		&CloseSocket();
    		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
                if ($argList[0] == $WA_SUCCESS) {
                    $VAR{'INFORMATION_VAR'} = &InfoStyle( $PASSWORD_CHANGED, "BLUE" );
                    &DisplayCachedModifyUser();
                }
                elsif ($argList[0] == $WA_WRONG_PASSWORD) {
                    $VAR{'INFORMATION_VAR'} = &InfoStyle( $PASSWORD_INCORRECT, "RED" );
                    &DisplayCachedModifyUser();
                }
                else {
                    &AppServerError( 'ChangePassword', $argList[0] );
                }
				$done = 1;
            }
    		else {
				$try++;
    		}
	    } continue {
			$try++;
		}

    	if($done eq 0) {
			&DaemonError( 'ChangePassword', $errCode );
    	}
    }
    else {
        # Some mandatory fields are missing.
        $list = "";
		if ($FORM{'OLD_PASSWORD'} eq '') {
			$list = "$list<B>$OLD_PASSWORD</B>, ";
		}
		if ($FORM{'NEW_PASSWORD'} eq '') {
			$list = "$list<B>$NEW_PASSWORD</B>, ";
		}
        if ($FORM{'CONFIRM_PASSWORD'} eq '') {
			$list = "$list<B>$CONFIRM_NEW_PASSWORD</B>, ";
		}
        chop $list; chop $list;  # Get rid of trailing space and comma.
        $VAR{'INFORMATION_VAR'} = &InfoStyle( "$MANDATORY_FIELDS $list", "RED" );
        &DisplayCachedModifyUser();
    }
    &LogItem("<-- ChangePassword");
}

# Needed for reuire statement in Main

1;
