#!/usr/local/bin/perl

#
# solve.pl
#
# Diagnose()
# &DisplayDiagInput()
# &DisplayDiagnostics()
# GetHomePages()
# HypernodeDetail()
# SolutionDetail()
# SolutionList()
#

###############################################################################
# Procedure:   &DisplayDiagnostics
#
# FUNC:        DISP_DIAGNOSTICS
#
# Description: Displays the diagnostics page, showing links and explanations
#              for the diagnostics the user has rights to use.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayDiagnostics {
    &LogItem("--> DisplayDiagnostics()");
	&ValidateUser();

    $VAR{'DIAGNOSTIC_LINKS_VAR'} = &GetDiagnosticTable();

	&SetAutoLogoutTag();
    &set_diag_home_strings();
	&EchoFile($HTML_PATH, "diag_home.htm");
    &LogItem("<-- DisplayDiagnostics()");
}

###############################################################################
# Procedure:   &DisplayDiagInput
#
# FUNC:        DISP_DIAG_INPUT
#
# Description: Displays the form input elements for the diagnostic type chosen.
#              The System combo box is always displayed.  If the diagnostic type
#              is Adaptive Learning, the description box is also displayed.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayDiagInput {
    &LogItem("--> DisplayDiagInput()");
    &ValidateUser();

    $VAR{'DIAGNOSTIC_TYPE_VAR'} = $FORM{'DIAGNOSTIC_TYPE'};
    &SystemBox();
    if ($FORM{'DIAGNOSTIC_TYPE'} == 1) {
        $VAR{'SOLUTION_TITLE_VAR'} = $ADAPTIVE_LEARNING;
        &DescriptionBox();
    }
    elsif ($FORM{'DIAGNOSTIC_TYPE'} == 2) {
        $VAR{'SOLUTION_TITLE_VAR'} = $COMMON_PROBLEMS;
    }
    elsif ($FORM{'DIAGNOSTIC_TYPE'} == 3) {
        $VAR{'SOLUTION_TITLE_VAR'} = $HOT_NEWS;
    }
    elsif ($FORM{'DIAGNOSTIC_TYPE'} == 4) {
        $VAR{'SOLUTION_TITLE_VAR'} = $ERROR_MESSAGES;
    }

	&SetAutoLogoutTag();
    &set_diag_input_strings();
    &EchoFile($HTML_PATH, "diag_input.htm");
    &LogItem("<-- DisplayDiagInput()");
}

###############################################################################
# Procedure:   Diagnose
#
# FUNC:        DIAGNOSE
#
# Description: Displays the appropriate homepages or a list of solutions,
#              depending on the diagnostic being used.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub Diagnose {
    &LogItem("--> Diagnose()");
    if ($FORM{'DIAGNOSTIC_TYPE'} == 5) {
        &GetHomePages();
    }
    else {
        &SolutionList();
    }
    &LogItem("<-- Diagnose()");
}

###############################################################################
# Procedure:   GetHomePages
#
# FUNC:        DIAGNOSE, GET_HOMEPAGE
#
# Description: Displays all the homepages associated with a particular
#              screen (i.e. $FORM{'PAGE_ID'}).  The default screen has page_id
#              of 1.  The links become links to other homepages or links to
#              hypernodes.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub GetHomePages {
    &LogItem("--> GetHomePages()");
	&ValidateUser();

	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_GetHomePages', $STRING);
		&AddArg($knowMask, $STRING);
        if ($FORM{'PAGE_ID'}) {
            $pageID = $FORM{'PAGE_ID'};
        }
        else {
            $pageID = '1';  # Default the first home page.
        }
        &AddArg($pageID, $STRING);
        &SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_NO_CALLER_CMD);
    	($cmd, $subCmd, $errCode) = &RecvMsg();
	    &CloseSocket();
    	if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
            if ($argList[0] > 0){
                $argIdx = 2;
                $pageList = '';
                for($i = 0; $i<$argList[0]; $i++){
                    $operation = $argList[$argIdx++];
                    $id = $argList[$argIdx++];
                    $desc = $argList[$argIdx++];
                    if ($operation == 1){
                        # This operation means the $id refers to another home page.
                        $pageList = "$pageList<TR><TD ALIGN=LEFT VALIGN=TOP WIDTH=\"100%\"><LI><A HREF=\"$SCRIPT_PATH/web_access.cgi?FUNC=GET_HOMEPAGE&PAGE_ID=$id\">$desc</A></TD></TR>\n";
                    }
                    elsif ($operation == 2){
                        # This operation means a system action will be executed.
                        # Since we have no way of doing this, we skip it.
                        # NOTE: Homepages with operation of 2 should be filtered by the
                        #       AppServer code so this code block should never execute.
                    }
                    else{
                        # This operation means the $id refers to a hypernode
                        $pageList = "$pageList<TR><TD ALIGN=LEFT VALIGN=TOP WIDTH=\"100%\"><LI><A HREF=\"$SCRIPT_PATH/web_access.cgi?FUNC=HYPERNODE_DETAIL&NODE_ID=$id\">$desc</A></TD></TR>\n";
                    }
                }
                $VAR{'HOMEPAGE_LIST_VAR'} = $pageList;
                $VAR{'HOMEPAGE_TITLE_VAR'} = $argList[1];
				&SetAutoLogoutTag();
                &set_hypernode_entry_strings();
                &EchoFile($HTML_PATH, "hypernode_entry.htm");
            }
            elsif ($argList[0] == 0) {
                #TODO: Change this to represent something decent.
				$VAR{'INFORMATION_VAR'} = &InfoStyle( $NO_HOME_PAGES, "RED" );
	            &DisplayDiagnostics();
	        }
            elsif ($argList[0] == $WA_INVALID_ID) {
                #TODO: Give a kind error message.
                &LogItem("! GetHomepages : Invalid ID = $pageID");
                $VAR{'INFORMATION_VAR'} = &InfoStyle( $INVALID_ID_ERROR, "RED" );
                &DisplayDiagnostics();
            }
            else {
                &AppServerError( 'GetHomePages', $argList[0] );
            }
			$done = 1;
    	}
        else{
			$try++;
        }
    } continue {
		$try++;
	}

    if($done eq 0) {
        &DaemonError( 'GetHomePages', $errCode );
	}
    &LogItem("<-- GetHomePages()");
}

###############################################################################
# Procedure:   SolutionList
#
# FUNC:        DIAGNOSE
#
# Description: Requests the database to match the search criteria provided, and
#              displays a list of links to solutions along with its description.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub SolutionList {
    &LogItem("--> SolutionList()");
    &ValidateUser();

    # Must do this first, before the other socket is created.
    &SystemBox();

	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_GetDiagAids', $STRING);
		&AddArg($knowMask, $STRING);
        &AddArg($FORM{'DIAGNOSTIC_TYPE'}, $STRING);
        &AddArg($FORM{'SYSTEM'}, $STRING);
		&AddArg($organizationID, $STRING);

		if( exists($FORM{'START_IDX'})) {
			$startIdx = $FORM{'START_IDX'};
		}
		else {
			$startIdx = 1;
		}

        if (!$FORM{'PARENT_ID'}) {
            $FORM{'PARENT_ID'} = -1;
        }
        $VAR{'DIAGNOSTIC_TYPE_VAR'} = $FORM{'DIAGNOSTIC_TYPE'};

        # Add type-specific arguments and set the page titles.
		if($FORM{'DIAGNOSTIC_TYPE'} == 1) {	# Adaptive Learning
            &AddArg($FORM{'DESCRIPTION'}, $STRING);
			$VAR{'SOLUTION_TITLE_VAR'} = $ADAPTIVE_LEARNING;
            &DescriptionBox();
		}
		elsif($FORM{'DIAGNOSTIC_TYPE'} == 2) {	# Common Problems
            &AddArg($FORM{'PARENT_ID'}, $STRING);
			$VAR{'SOLUTION_TITLE_VAR'} = $COMMON_PROBLEMS;
		}
		elsif($FORM{'DIAGNOSTIC_TYPE'} == 3) {	# Hot News
            &AddArg($FORM{'PARENT_ID'}, $STRING);
			$VAR{'SOLUTION_TITLE_VAR'} = $HOT_NEWS;
		}
		elsif($FORM{'DIAGNOSTIC_TYPE'} == 4) {	# Error Messages
            &AddArg($FORM{'PARENT_ID'}, $STRING);
			$VAR{'SOLUTION_TITLE_VAR'} = $ERROR_MESSAGES;
		}
		else {
			&Error(); # Unexpected error.
		}
        &AddArg($startIdx, $STRING);
        &AddArg($MAX_DIAG, $STRING);

        &SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_NO_CALLER_CMD);
        ($cmd, $subCmd, $errCode) = &RecvMsg();
        &CloseSocket();
        if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
			$argIdx = 1;
			$solList = "";
			if($argList[0] > 0) {
				for($i=0; $i < $argList[0]; $i++) {
					$solList = "$solList<TR>\n";
                    if ($FORM{'DIAGNOSTIC_TYPE'} == 1) { # Adaptive Learning
                        $argList[$argIdx++];  # Ignore is_group flag.
                        $solID = $argList[$argIdx++];
                        $score = $argList[$argIdx++];
                        $desc  = &FormatLinkedString( $argList[$argIdx++] );
                        $solList = "$solList<TD ALIGN=LEFT VALIGN=TOP WIDTH=\"100%\"><LI><A HREF=\"$SCRIPT_PATH/web_access.cgi?FUNC=SOLUTION_DETAIL&SOL_ID=$solID\">($SCORE: $score) $desc</A></TD>\n"; # Solution
                    }
                    elsif ($argList[$argIdx++] == 2) { # is_group == 2, grouped aid
                        $parentID = $argList[$argIdx++];
                        $desc  = &FormatLinkedString( $argList[$argIdx++] );
                        $solList = "$solList<TD ALIGN=LEFT VALIGN=TOP WIDTH=\"100%\"><LI><A HREF=\"$SCRIPT_PATH/web_access.cgi?FUNC=DIAGNOSE&DIAGNOSTIC_TYPE=$FORM{'DIAGNOSTIC_TYPE'}&PARENT_ID=$parentID\">$desc</A></TD>\n";
                    }
                    else { # Common Problem, Error Message, or Hot News solution.
                        $solID = $argList[$argIdx++];
                        $desc  = &FormatLinkedString( $argList[$argIdx++] );
                        $solList = "$solList<TD ALIGN=LEFT VALIGN=TOP WIDTH=\"100%\"><LI><A HREF=\"$SCRIPT_PATH/web_access.cgi?FUNC=SOLUTION_DETAIL&SOL_ID=$solID\">$desc</A></TD>\n"; # Solution
                    }
					$solList = "$solList</TR>\n";
				}
	            $VAR{'SOLUTION_LIST_VAR'} = $solList;

                $totalMatches = $argList[$argIdx];
                if ($startIdx > 1) {
                    $prev = 1;
                }
                else {
                    $prev = 0;
                }
                if (($startIdx + $MAX_DIAG - 1) < $totalMatches)  {
                    $next = 1;
                }
                else {
                    $next = 0;
                }

				if(exists($FORM{'NEXT'})) {
					$VAR{'PAGE_NUMBER_VAR'} = $FORM{'PAGE_NUMBER'} + 1;
				}
				elsif(exists($FORM{'PREV'})) {
					$VAR{'PAGE_NUMBER_VAR'} = $FORM{'PAGE_NUMBER'} - 1;
				}
				else {
					$VAR{'PAGE_NUMBER_VAR'} = 1;
				}

                $VAR{'TOTAL_MATCHES_VAR'} = $totalMatches;
                $tmp = $totalMatches / $MAX_DIAG;

                # Round the total pages up.
                $VAR{'TOTAL_PAGES_VAR'} = ($tmp > (int $tmp)) ? (int $tmp) + 1 : (int $tmp);
				$VAR{'PREV_NEXT_BUTTONS_VAR'} = &PrevNextButtons($startIdx, $prev, $next);

				&SetAutoLogoutTag();
                &set_sol_list_strings();
	            &EchoFile($HTML_PATH, "sol_list.htm");
			}
			elsif ($argList[0] == 0) {
                # No solutions matched the criteria.  Redisplay the search criteria page.
				$VAR{'INFORMATION_VAR'} = &InfoStyle( $NO_SOLUTIONS, "RED" );
	            &DisplayDiagInput();
			}
            else {
                &AppServerError( 'SolutionList', $argList[0] );
            }
			$done = 1;
		}
		else {
			$try++;
		}
    } continue {
		$try++;
	}

	if($done eq 0) {
		&DaemonError( 'SolutionList', $errCode );
	}
    &LogItem("<-- SolutionList()");
}

###############################################################################
# Procedure:   HypernodeDetail
#
# FUNC:        HYPERNODE_DETAIL
#
# Description: Displays detailed information about a particular hypernode, showing
#              its title and action for all nodes.  For solved nodes, it gives
#              the solution detail as well.  For unsolved nodes, it gives the
#              questions and a series of responses.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub HypernodeDetail {
    &LogItem("--> HypernodeDetail()");
    &ValidateUser();

    $nodeID = $FORM{'NODE_ID'};

	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_HypernodeDetail', $STRING);
        &AddArg($knowMask, $STRING);
        &AddArg($nodeID, $STRING);
        &SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_NO_CALLER_CMD);
        ($cmd, $subCmd, $errCode) = &RecvMsg();
        &CloseSocket();
        if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
            if ($argList[0] == $WA_SUCCESS) {
                $argIdx = 1;
                $VAR{'NODETITLE_VAR'} = $argList[$argIdx++];
                $VAR{'ACTION_VAR'} = &FormatString( $argList[$argIdx++] );
                $solved = $argList[$argIdx++];
                if ($solved ne 'TRUE') {
                    # This is a question/response node.  Show the question and
                    # a list of responses.
                    $VAR{'QUESTION_VAR'} = &FormatString( $argList[$argIdx++] );
                    $numResponses = $argList[$argIdx++];
                    $respList = "";
                    for ($i = 0; $i<$numResponses; $i++) {
                        $respList = "$respList<TR><TD ALIGN=LEFT><A HREF=\"$SCRIPT_PATH/web_access.cgi?FUNC=HYPERNODE_DETAIL&NODE_ID=$argList[$argIdx++]\">$argList[$argIdx++]</A></TD></TR>\n"; # Response
                    }
                    $VAR{'RESPONSE_LIST_VAR'} = $respList;
					&SetAutoLogoutTag();
                    &set_hypernode_strings();
                    &EchoFile($HTML_PATH, "hypernode.htm");
                }
                else
                {
                    # This is a solved node.  Show the details of the solution.
   				    $VAR{'PROBLEM_TYPE_VAR'} = $argList[$argIdx++];
    				$dateTime = &FormatDateTimeStr($argList[$argIdx++], $argList[$argIdx++]);
	    			$VAR{'DATE_TIME_ADDED_VAR'} = $dateTime;
		    		$VAR{'DESCRIPTION_VAR'} = &FormatString( $argList[$argIdx++] );
			    	$VAR{'SOLUTION_VAR'} = &FormatString( $argList[$argIdx++] );
                    $VAR{'SOLUTION_TITLE_VAR'} = $VAR{'NODETITLE_VAR'};
                    $VAR{'NODE_INFORMATION_VAR'} = $_NODE_INFORMATION_TAG;
					&SetAutoLogoutTag();
                    &set_sol_detail_strings();
                    &EchoFile($HTML_PATH, "sol_detail.htm");
                }
            }
            elsif ($argList[0] == $WA_INVALID_ID) {
                &LogItem("! HypernodeDetail : Invalid ID = $nodeID");
                $VAR{'INFORMATION_VAR'} = &InfoStyle( $INVALID_ID_ERROR, "RED" );
                &DisplayDiagnostics();
            }
            elsif ($argList[0] == $WA_KNOWLEDGE_RIGHT_FAILED) {
                &LogItem("! HypernodeDetail : Knowledge Right breach");
                $VAR{'INFORMATION_VAR'} = &InfoStyle( $KNOWLEDGE_RIGHT_ERROR, "RED" );
                &DisplayDiagnostics();
            }
            else{
                &AppServerError( 'HypernodeDetail', $argList[0] );
            }
			$done = 1;
        }
        else{
			$try++;
        }
    } continue {
		$try++;
	}

    if($done eq 0) {
        &DaemonError( 'HypernodeDetail', $errCode );
    }
    &LogItem("<-- HypernodeDetail()");
}

###############################################################################
# Procedure:   SolutionDetail
#
# FUNC:        SOLUTION_DETAIL
#
# Description: Displays detailed information about a specific solution.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub SolutionDetail {
    &LogItem("--> SolutionDetail()");
    &ValidateUser();

	$solutionID = $FORM{'SOL_ID'};

	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_SolutionDetail', $STRING);
		&AddArg($knowMask, $STRING);
		&AddArg($solutionID, $STRING);
		&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_NO_CALLER_CMD);
		($cmd, $subCmd, $errCode) = &RecvMsg();
		&CloseSocket();
		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
			if($argList[0] == $WA_SUCCESS) {
				$argIdx = 1;
				$VAR{'PROBLEM_TYPE_VAR'} = $argList[$argIdx++];
				$dateTime = &FormatDateTimeStr($argList[$argIdx++], $argList[$argIdx++]);
				$VAR{'DATE_TIME_ADDED_VAR'} = $dateTime;
				$VAR{'DESCRIPTION_VAR'} = &FormatString( $argList[$argIdx++] );
				$VAR{'SOLUTION_VAR'} = &FormatString( $argList[$argIdx++] );
                $VAR{'SOLUTION_TITLE_VAR'} = $SOLUTION;
				&SetAutoLogoutTag();
                &set_sol_detail_strings();
				&EchoFile($HTML_PATH, "sol_detail.htm");
			}
            elsif ($argList[0] == $WA_INVALID_ID) {
                &LogItem("! HypernodeDetail : Invalid ID = $solutionID");
                $VAR{'INFORMATION_VAR'} = &InfoStyle( $INVALID_ID_ERROR, "RED" );
                &DisplayDiagnostics();
            }
            elsif ($argList[0] == $WA_KNOWLEDGE_RIGHT_FAILED) {
                &LogItem("! SolutionDetail : Knowledge Right breach");
                $VAR{'INFORMATION_VAR'} = &InfoStyle( $KNOWLEDGE_RIGHT_ERROR, "RED" );
                &DisplayDiagnostics();
            }
			else {
                &AppServerError( 'SolutionDetail', $argList[0] );
			}
			$done = 1;
		}
		else {
			$try++
		}
    } continue {
		$try++;
	}

	if($done eq 0) {
		&DaemonError( 'SolutionDetail', $errCode );
	}
    &LogItem("<-- SolutionDetail()");
}

# Needed for reuire statement in Main

1;
