#!/usr/local/bin/perl

#
# web_access.cgi
#
# This file is the main file for the TPM Web Access project.
#

# Require routines necessary for this script to run.

# Load the configuration file
require 'web_access.cfg';

# These files have some standard routines and constants
require 'strings.pl';
require 'commands.pl';
require 'html.pl';
require 'datetime.pl';
require 'error.pl';
require 'cookie.pl';
require 'sockmsg.pl';
require 'logfile.pl';
require 'setvars.pl';

# These files have the functions directly called below
require 'login.pl';
require 'solve.pl';
require 'submit.pl';
require 'view.pl';
require 'modify.pl';
require 'misc.pl';

# Set up some global variables
$MAX_RETRIES = 3;
$CookieSet = 0;
$VAR{'SCRIPT_PATH'} = $SCRIPT_PATH;
$KML_FILE = 'webacces';

$ADAPTIVE_LEARNING_BIT = 1 << 0;
$COMMON_PROBLEMS_BIT   = 1 << 3;
$HOT_NEWS_BIT          = 1 << 2;
$ERROR_MESSAGES_BIT    = 1 << 4;
$HYPER_TREES_BIT       = 1 << 1;

# This variable is used extensively to keep track of parameters
# to the CGI.
%FORM = ();

# Parse the parameters
if(&ParseForm()) {
	&LogItem("*** Command Received: $FORM{'FUNC'} ***");

    if(&GetCompressedCookies('TPM-WA')) {
		$userID         = $Cookies{'UserID'};
		$userAuthID     = $Cookies{'UserAuthID'};
		$cookieTime     = $Cookies{'CookieTime'};
        $contactID      = $Cookies{'ContactID'};
        $organizationID = $Cookies{'OrganizationID'};
		$createRight    = $Cookies{'Create'};
		$viewRight      = $Cookies{'View'};
		$updateRight    = $Cookies{'Update'};
		$diagRight      = $Cookies{'Diag'};
        $a_createRight  = $Cookies{'A_Create'};
        $a_viewRight    = $Cookies{'A_View'};
		$a_updateRight  = $Cookies{'A_Update'};
		$a_diagRight    = $Cookies{'A_Diag'};
        $knowMask       = $Cookies{'KnowMask'};

	    &LogItem("userID - $userID ");
	    &LogItem("userAuthID - $userAuthID ");
	    &LogItem("cookieTime - $cookieTime ");
	    &LogItem("contactID - $contactID ");
	    &LogItem("organizationID - $organizationID ");
	    &LogItem("createRight - $createRight ");
	    &LogItem("viewRight - $viewRight ");
	    &LogItem("updateRight - $updateRight ");
	    &LogItem("diagRight - $diagRight ");
	    &LogItem("a_createRight - $a_createRight ");
	    &LogItem("a_viewRight - $a_viewRight ");
	    &LogItem("a_updateRight - $a_updateRight ");
	    &LogItem("a_diagRight - $a_diagRight ");
	    &LogItem("knowMask - $knowMask ");

        # Build the footer that will go on every screen.
        &BuildFooter();
	}
	elsif(($FORM{'FUNC'} eq 'DISP_NEW_USER')||($FORM{'FUNC'} eq 'CREATE_NEW_USER')||
	      ($FORM{'FUNC'} eq 'DISP_REMINDER')||($FORM{'FUNC'} eq 'REMINDER')) {
        # Do nothing because it is not an error
        # if the user is new or hasn't logged in yet
        # and there are no cookies on the system.
	}
	else {
        # If there are no cookies, and the user is not already logging in,
        # display the login screen because the user has not logged in yet.

		if(($FORM{'FUNC'} ne 'LOGIN')&&($FORM{'FUNC'} ne 'ANONYMOUS_LOGIN')) {
			&StartApp();
            exit;
		}
	}

    # The login function is special because it can result in several
    # screens being shown, due to long periods of use inactivity.

	if ($FORM{'FUNC'} eq 'LOGIN') {
        # If $FORM{'LAST'} is not empty, we know that this is a "timeout" login,
        # so we use $FORM{'LAST'} as the new $FORM{'FUNC'} for this interaction.
		if( (&Login() == 1) && ($FORM{'LAST'} ne '') ) {
			$FORM{'FUNC'} = $FORM{'LAST'};
            &LogItem("Timeout login okay: New func = $FORM{'FUNC'}");
		}
        else {
            # Either the login failed, or this was a normal login.
            # Just exit, because the Login() function took care of
            # all necessary actions
            exit;
        }
	}

    #
    # Determine what function the user wants
    #

	if ($FORM{'FUNC'} eq 'DISP_LOGIN') {            # Show the login screen
		&DisplayLogin();
	}
    elsif ($FORM{'FUNC'} eq 'LOGIN') {              # Perform the login
		&Login();
	}
    elsif ($FORM{'FUNC'} eq 'ANONYMOUS_LOGIN') {    # Perform anonymous login
        &AnonymousLogin();
    }
	elsif ($FORM{'FUNC'} eq 'LOGOUT') { 			# Logout of App
		&Logout();
	}
    elsif ($FORM{'FUNC'} eq 'ENTRY') {              # Entry - Special page used for security
		&Entry();
	}
	elsif ($FORM{'FUNC'} eq 'DISP_MENU') {          # Display Frame
		&DisplayMenu();
	}
	elsif ($FORM{'FUNC'} eq 'DISP_HOMEPAGE') {      # Display Homepage
		&DisplayHomePage();
	}
	elsif ($FORM{'FUNC'} eq 'DISP_SUBMIT') { 		# Display Submit Screen
        &RightViolation($CREATE_RIGHT) if (($createRight == 0) && ($a_createRight == 0));
		&DisplaySubmit();
	}
	elsif ($FORM{'FUNC'} eq 'SUBMIT') {				# Submit the problem; display problem details
        &RightViolation($CREATE_RIGHT) if (($createRight == 0) && ($a_createRight == 0));
    	&Submit();
	}
	elsif ($FORM{'FUNC'} eq 'DISP_PROB_INQUIRY') {  # Display Problem Inquiry screen
        &RightViolation($VIEW_RIGHT) if ($viewRight == 0);
		&DisplayProblemInquiry();
	}
	elsif ($FORM{'FUNC'} eq 'PROB_INQUIRY') {       # Inquiry Specific Problem Screen
        &RightViolation($VIEW_RIGHT) if ($viewRight == 0);
		&ProblemInquiry();
	}
	elsif ($FORM{'FUNC'} eq 'PROB_LIST') {          # Inquiry Problem List Screen
        &RightViolation($VIEW_RIGHT) if ($viewRight == 0);
		&ProblemInquiry();
	}
	elsif ($FORM{'FUNC'} eq 'PROB_DETAIL') {        # Problem Detail Screen
        &RightViolation($VIEW_RIGHT) if (($viewRight == 0) && ($a_viewRight == 0));
		&ProblemDetail();
	}
	elsif ($FORM{'FUNC'} eq 'DISP_UPDATE') { 		# Display Update Problem Screen
        &RightViolation($UPDATE_RIGHT) if (($updateRight == 0) && ($a_updateRight == 0));
		&DisplayUpdate();
	}
	elsif ($FORM{'FUNC'} eq 'UPDATE') {				# Update Problem
        &RightViolation($UPDATE_RIGHT) if (($updateRight == 0) && ($a_updateRight == 0));
		&Update();
	}
	elsif ($FORM{'FUNC'} eq 'VIEW_PROB_HISTORY') {  # Display Problem History Screen
        &RightViolation($VIEW_RIGHT) if (($viewRight == 0) && ($a_viewRight == 0));
		&GetProblemHistory();
	}
	elsif ($FORM{'FUNC'} eq 'VIEW_WORK_HISTORY') { 	# Display Work History Screen
        &RightViolation($VIEW_RIGHT) if (($viewRight == 0) && ($a_viewRight == 0));
		&GetWorkHistory();
	}
    elsif ($FORM{'FUNC'} eq 'DISP_DIAGNOSTICS') {   # Display Diagnostic Choice Screen
        &RightViolation($DIAG_RIGHT) if (($diagRight == 0) && ($a_diagRight == 0));
        &DisplayDiagnostics();
    }
    elsif ($FORM{'FUNC'} eq 'DISP_DIAG_INPUT') {    # Display Diagnostic Input Screen
        &RightViolation($DIAG_RIGHT) if (!&VerifyDiagRight());
        &DisplayDiagInput();
    }
    elsif ($FORM{'FUNC'} eq 'DIAGNOSE') {           # Diagnose the problem
        &RightViolation($DIAG_RIGHT) if (!&VerifyDiagRight());
        &Diagnose();
    }
    elsif ($FORM{'FUNC'} eq 'GET_HOMEPAGE') {       # Hypernode Homepage
        &RightViolation($DIAG_RIGHT) unless (($diagRight & $HYPER_TREES_BIT) || ($a_diagRight & $HYPER_TREES_BIT));
        &GetHomePages();
    }
    elsif ($FORM{'FUNC'} eq 'HYPERNODE_DETAIL') {   # Hypernode Details
        &RightVioation($DIAG_RIGHT) unless (($diagRight & $HYPER_TREES_BIT) || ($a_diagRight & $HYPER_TREES_BIT));
        &HypernodeDetail();
    }
	elsif ($FORM{'FUNC'} eq 'SOLUTION_DETAIL') { 	# Solution Details
        &RightViolation($DIAG_RIGHT) if (($diagRight == 0) && ($a_diagRight == 0));
		&SolutionDetail();
	}
	elsif ($FORM{'FUNC'} eq 'DISP_USER_CFG') {      # Display User Cfg Screen
        &RightViolation() if ($userID eq '');
		&DisplayModifyUser();
	}
	elsif ($FORM{'FUNC'} eq 'MODIFY_USER') {        # Modify User
        &RightViolation() if ($userID eq '');
		&ModifyUser();
	}
    elsif ($FORM{'FUNC'} eq 'CHANGE_PASSWORD') {    # Change Password
        &RightViolation() if ($userID eq '');
		&ChangePassword();
	}
	elsif ($FORM{'FUNC'} eq 'DISP_NEW_USER') { 		# Display New User Screen
		&DisplayNewUser();
	}
	elsif ($FORM{'FUNC'} eq 'CREATE_NEW_USER') {    # Create a New User
		&CreateNewUser();
	}
    elsif ($FORM{'FUNC'} eq 'DISP_REMINDER') {      # Display Reminder Screen
        &DisplayReminder();
    }
    elsif ($FORM{'FUNC'} eq 'REMINDER') {           # Send UserID and Password
        &Reminder();
    }
	else {
		&Error($INVALID_COMMAND, $INVALID_COMMAND_TITLE, $INVALID_COMMAND_TITLE);
	}
}
else {
    &LogItem("! ParseForm() failed!");
    &Error(); # Unexpected error!
}

exit;
